from openmm import app, unit
from openmmtools import alchemy
from openmmtools import mcmc, multistate, states

# Load the topology and coordinate files.
top = app.AmberPrmtopFile('OpenMMTools.prm7')
positions = app.AmberInpcrdFile('OpenMMTools.rst7')

# Initialise the molecular system.
mm_system = top.createSystem(
    nonbondedMethod=app.NoCutoff,
    constraints=app.HBonds,
)

# Initialise the alchemical system.
alchemical_atoms=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25]
alchemical_region = alchemy.AlchemicalRegion(alchemical_atoms=alchemical_atoms)
factory = alchemy.AbsoluteAlchemicalFactory()
alchemical_system = factory.create_alchemical_system(mm_system, alchemical_region)
alchemical_state = alchemy.AlchemicalState.from_system(alchemical_system)

# Define compound thermodynamic states.
protocol = {
    'lambda_electrostatics': [1.00000],
    'temperature': [300.0 * unit.kelvin] * 1,
    'pressure': [None] * 1,
}
compound_thermodynamic_states = states.create_thermodynamic_state_protocol(
    alchemical_system, protocol=protocol, composable_states=[alchemical_state]
)

# Define the integrator.
move = mcmc.LangevinDynamicsMove(
    timestep=2.0 * unit.femtosecond,
    collision_rate=1.0 / unit.picoseconds,
    n_steps=1000,
)

# Initialise and configure the simulation object.
simulation = multistate.MultiStateSampler(
    mcmc_moves=move,
    number_of_iterations=2,
    online_analysis_interval=None,
)

# Add reporters.
reporter = multistate.MultiStateReporter(
    'OpenMMTools.nc',
    checkpoint_interval=1,
)

# Add sampler.
sampler_states = states.SamplerState(
    positions.positions,
)
simulation.create(
    thermodynamic_states=compound_thermodynamic_states,
    sampler_states=sampler_states,
    storage=reporter,
    initial_thermodynamic_states=[0],
)
simulation.run()
